#!/usr/bin/env python
#
# $Id: status.py,v 1.2 2006/12/05 13:10:45 doughellmann Exp $
#
# Copyright 2006 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Module interface for providing status messages to the user.

"""

#
# Import system modules
#


#
# Import Local modules
#


#
# Module
#

_status_message_func = None

def statusMessage(msg, verboseLevel=1, error=None):
    """Print a status message to output.

    Arguments

        msg=''            -- The status message string to be printed.

        verboseLevel=1    -- The verbose level to use.  The message
                          will only be printed if the current verbose
                          level is >= this number.

        error=None        -- If true, the message is considered an error and
                          printed as such.

    """
    global _status_message_func
    if _status_message_func:
        _status_message_func(msg, verboseLevel, error)
    return

def registerStatusMessageFunc(function):
    """Register the function to be called by statusMessage().
    """
    global _status_message_func
    _status_message_func = function
    return
    
